/**
 * RightJS Custom element wrappers module
 * <url here>
 *
 * Copyright (C) 2010 Alexey Dubinin <LemmingKing at ya dot ru>
 */
(function(RightJS) {
  
/**
 * The plugin initializtion script
 *
 * Copyright (C) 2010 Nikolay Nemshilov
 */
 
var R        = RightJS,
    $        = RightJS.$,
    $w       = RightJS.$w,
    $A       = RightJS.$A,
    Class    = RightJS.Class,
    Object   = RightJS.Object,
    Element  = RightJS.Element,
    Wrapper  = RightJS.Wrapper,
    defined  = RightJS.defined,
    isHash   = RightJS.isHash,
    isString = RightJS.isString;

/**
 * Prepare custom wrappers storage
 */
Element.CSSWrappers = null;
Element.ClassWrappers = null;

/**
 * Searches for an appropriate wrapper
 *
 * When calling the $ function, wrapper is choosed in the following way:
 *  1 - watch through the array of CSSWrappers in reverse order (so that wrappers registered later have privilege
 *  2 - if (1) brings no result, watch element's classes and use the first found wrapper (ClassWrappers are unordered)
 *  3 - finally try to find TAG-only wrapper in the Wrappers set
 *
 * @param DomElement unit
 */
Wrapper.Cast = function(unit) {
    var i;
    if(Element.CSSWrappers) {
        for(i=Element.CSSWrappers.length-1; i>=0; i--) {
            if(_match(unit, Element.CSSWrappers[i].selector)) {
                return Element.CSSWrappers[i].wrapper;
            }
        }
    }
    if(Element.ClassWrappers) {
        var classes = unit.className.trim().split(/\s+/), key;
        for(i=0; i<classes.length; i++) {
            key = "." + classes[i];
            if(unit.tagName + key in Element.ClassWrappers) {
                return Element.ClassWrappers[unit.tagName + key];
            }
            if(key in Element.ClassWrappers) {
                return Element.ClassWrappers[key];
            }
        }
    }
    return (unit.tagName in Element.Wrappers) ? Element.Wrappers[unit.tagName] : undefined;
};

/**
 * Registers wrapper to use with the $ function.
 * You can register it for:
 *  - a full-featured CSS-selector
 *  - simple and fast "TAG.class" selector or just ".class"
 *  - "TAG" only
 *
 * You can call this function in the following ways:
 *  RegisterWrapper(wrp, 'selector')                     - auto detect selector type
 *  RegisterWrapper(wrp, 'selector', {options})          - set options
 *  RegisterWrapper(wrp, {selector:'selector', options}) - another variant of the previous call
 * 
 * The only valid option is:
 *  Bool force_css - if true, force registering the selector as slow full-featured CSS-selector
 *
 * NOTE: registering full-featured CSS-selectors will significally decrease performance
 *
 * @param Wrapper wrapper to register
 * @param String selector
 * @param Object options optional
 */
RightJS.RegisterWrapper = function(wrapper, selector, options) {
    if(isHash(selector) && typeof(options) === 'undefined') {
        options  = selector;
        selector = options.selector;
    }
    var arr = /^(\w+)?(\.[\w\-\._]+)?$/.exec(selector.trim());
    if(arr === null || (defined(options) && options.force_css)) {
        (Element.CSSWrappers || (Element.CSSWrappers = [])).push({'wrapper':wrapper, 'selector':selector});
    } else if(typeof(arr[2]) === 'undefined' || arr[2] === '' /* IE 8 hack */) {
        Element.Wrappers[arr[1].toUpperCase()] = wrapper;
    } else if(typeof(arr[1]) === 'undefined' || arr[1] === '' /* IE 8 hack */) {
        (Element.ClassWrappers || (Element.ClassWrappers = {}))[arr[2]] = wrapper;
    } else {
        (Element.ClassWrappers || (Element.ClassWrappers = {}))[arr[1].toUpperCase() + arr[2]] = wrapper;
    }
};

/**
 * Unregisters this wrapper. Slow, but rare operation
 *
 * @param Wrapper wrapper to unregister
 */
RightJS.UnregisterWrapper = function(wrapper) {
    if(Element.CSSWrappers) {
        Element.CSSWrappers = Element.CSSWrappers.without(wrapper);
    }
    if(Element.ClassWrappers) {
        for(c in Element.ClassWrappers) {
            if(Element.ClassWrappers[c] === wrapper) {
                delete Element.ClassWrappers[c];
                break;
            }
        }
    }
    for(c in Element.Wrappers) {
        if(Element.Wrappers[c] === wrapper) {
            delete Element.Wrappers[c];
            break;
        }
    }
};

// private

/**
 * Checks if native dom-element matches the rule.
 * We have to use native querySelectorAll because
 * we have not created Element yet
 *
 * @param DomElement unit
 * @param String css_rule
 */
_match = function(unit, css_rule) {
    var parent;
    if(unit.tagName === 'HTML') {
        parent = unit.ownerDocument;
    } else {
        parent = unit;
        while(parent.parentNode) {
            parent = parent.parentNode;
        }
    }
    // parent cannot be null
    return $A(parent.querySelectorAll(css_rule)).include(unit);
};

})(RightJS);
